﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win;

namespace PartiallyVisibleRowsSample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            //Populate grid with test data
            for (int i = 0; i < 21; i++)
            {
                typedDataSet1.DataTable1.Rows.Add("Column1Row:" + i, "Column2Row:" + i, "Column3Row:" + i, "Column4Row:" + i,
                    "Column5Row:" + i);
            }

            //Hook into DoubleClickRow Event
            ultraGrid1.DoubleClickRow += ultraGrid1_DoubleClickRow;

            //Create new instance of the CustomSelectionStrategyFilter class; pass in your specific grid instance here
            ultraGrid1.SelectionStrategyFilter = new CustomSelectionStrategyFilter(this.ultraGrid1);

            //When a cell is clicked, handle the action for the row so that the cell does not enter edit mode
            ultraGrid1.DisplayLayout.Override.CellClickAction = CellClickAction.RowSelect;
        }

        //Display pop-up window that indicates the row that has been clicked on
        void ultraGrid1_DoubleClickRow(object sender, Infragistics.Win.UltraWinGrid.DoubleClickRowEventArgs e)
        {
            MessageBox.Show(this, string.Format("Double Click Row: {0}", e.Row.VisibleIndex), "Double Click", MessageBoxButtons.OK);
        }
    }

    //Implement the ISelectionStrategyFilter interface on a class
    // (in this case the CustomSelectionStrategyFilter class)

    public class CustomSelectionStrategyFilter : ISelectionStrategyFilter
    {
        //Implement ISelectionStrategy
        //Default to null
        private ISelectionStrategy cellStrategy = null;

        //This method is intended to work only with the grid
        public CustomSelectionStrategyFilter(UltraGrid grid)
        {
            cellStrategy = new CustomSelectionStrategy(grid);
        }

        //In particular, we are only interested in inhibiting the scrolling effect, if the item in question is an UltraGridRow
        public ISelectionStrategy GetSelectionStrategy(Infragistics.Shared.ISelectableItem item)
        {
            if (item is UltraGridRow)
                return this.cellStrategy;

            return null;
        }
    }

    //Implement a custom class so that we can override certain features of the SelectionStrategyExtended class
    public class CustomSelectionStrategy : SelectionStrategyExtended
    {
        //Enforce relationship between CustomSelectionStrategy and ISelectionManger; reduce probability of error
        public CustomSelectionStrategy(ISelectionManager selectionManager) : base(selectionManager)
        {

        }

        public override bool OnMouseDown(Infragistics.Shared.ISelectableItem item, ref MouseMessageInfo msginfo)
        {
            UltraGrid grid = (SelectionManager as UltraGrid);

            if (grid == null)
                return base.OnMouseDown(item, ref msginfo);

            //On mousedown retrieve the UIElement at the mouse position 
            UIElement element = grid.DisplayLayout.UIElement.ElementFromPoint(new Point(msginfo.X, msginfo.Y));
            UIElement rowElement = null;

            CheckUIElementAndParents<RowUIElement>(element, ref rowElement);
            
            //Check to see if item is both a RowUIElement and that it is partially visible
            if (rowElement is RowUIElement && !rowElement.IsFullyVisible)
            {
                // At this point, we know that the user clicked on a Row that is not fully visible.
                SelectionManager.UnselectItem(item, false);
                grid.Selected.Rows.Clear();
                grid.ActiveRow = null;
                UltraGridRow row = rowElement.GetContext(typeof(UltraGridRow)) as UltraGridRow;
                grid.Selected.Rows.Add(row);
            }
            else
            {
                //If the row is not partially visible, handle the mousedown normally
                return base.OnMouseDown(item, ref msginfo);
            }

            return true;
        }

        //Check item that was clicked on to determine if it is a RowUIElement; pass reference back if it is
        private void CheckUIElementAndParents<T>(Infragistics.Win.UIElement uIElement, ref Infragistics.Win.UIElement outElement)
            //Place a constraint on CheckUIElementAndParents, so that only items that are UIElements are considered
            where T : Infragistics.Win.UIElement
        {
            //Check if the particular element is the specified type
            if (uIElement is T)
                //If it is, cast it to this specific type 
                outElement = uIElement as T;
                //If it is not, then check to see if this item has a parent, if so reference that parent
            else if (uIElement.Parent != null)
                CheckUIElementAndParents<T>(uIElement.Parent, ref outElement);
                //If none of these conditions are met, then do nothing
            else
                outElement = null;
        }
    }
}